package cards.maumau.model;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * Handles players in a MauMau game.
 */
class PlayerHandler {
    private final MauMau game;
    private final List<Player> players = new LinkedList<>();
    private final List<Player> ranking = new ArrayList<>();
    private Player remember;

    private interface PlayerState {
        void nextTurn(int n);
        void mau(Player p);
        void maumau(Player p);
    };

    private final  PlayerState waitForNextTurnState = new PlayerState() {

        @Override
        public void nextTurn(int n) {
            decide(n);
        }

        @Override
        public void mau(Player p) {
            //Do nothing
        }

        @Override
        public void maumau(Player p) {
            // Do nothing
        }

    };

    private final  PlayerState waitForMauState = new PlayerState() {

        @Override
        public void nextTurn(int n) {
            remember.drawCards(1);
            decide(n);
        }

        @Override
        public void mau(Player p) {
            if (p == remember) {
                currentState = waitForNextTurnState;
            }
        }

        @Override
        public void maumau(Player p) {
            //Do nothing
        }

    };

    private final  PlayerState waitForMauMauState = new PlayerState() {

        @Override
        public void nextTurn(int n) {
            remember.drawCards(1);
            decide(n);
        }

        @Override
        public void mau(Player p) {
            //Do nothing
        }

        @Override
        public void maumau(Player p) {
            if (p == remember) {
                finishPlayer(p);
            }
            if (players.size() == 1) {
                finishPlayer(getCurrentPlayer());
                game.getActionHandler().finishGame();
                currentState = finishedState;
            } else {
                currentState = waitForNextTurnState;
            }
        }

    };

    private final  PlayerState finishedState = new PlayerState() {

        @Override
        public void nextTurn(int n) {
            //Do nothing
        }

        @Override
        public void mau(Player p) {
            // Do nothing
        }

        @Override
        public void maumau(Player p) {
            // Do nothing
        }

    };

    private void decide(int n) {
        if (getCurrentPlayer().getCards().isEmpty()) {
            remember = getCurrentPlayer();
            localNextTurn(n);
            currentState = waitForMauMauState;
        } else if (getCurrentPlayer().getCards().size() == 1) {
            remember = getCurrentPlayer();
            localNextTurn(n);
            currentState = waitForMauState;
        } else {
            localNextTurn(n);
        }
    }

    private PlayerState currentState = waitForNextTurnState;

    /**
     * Constructs a PlayerHandler for the specified MauMau game.
     *
     * @param game The MauMau game instance.
     */
    PlayerHandler(MauMau game) {
        this.game = game;
    }

    /**
     * Initiates the next turn in the game.
     *
     * @param n The number of turns to proceed.
     */
    void nextTurn(int n) {
        currentState.nextTurn(n);
    }

    /**
     * Handles a player calling "Mau".
     *
     * @param p The player calling "Mau".
     */
    void mau(Player p) {
        currentState.mau(p);
    }

    /**
     * Handles a player calling "Mau-Mau".
     *
     * @param p The player calling "Mau-Mau".
     */
    void maumau(Player p) {
        currentState.maumau(p);
    }

    /**
     * Returns the list of players participating in the game.
     *
     * @return The list of players.
     */
    List<Player> getPlayers() {
        return players;
    }

    /**
     * Returns the ranking of players based on the order they finished the game.
     *
     * @return The ranking of players.
     */
    List<Player> getRanking() {
        return ranking;
    }

    /**
     * Adds a player to the game.
     *
     * @param player The player to add.
     * @throws IllegalArgumentException if a player with the same name already exists.
     */
    void addPlayer(Player player) {
        if (!players.contains(player)) {
            throw new IllegalArgumentException("Player with name " + player.getName() + " already exists.");
        }
        players.add(player);
    }

    /**
     * Moves to the next player's turn in the game.
     *
     * @param n The number of turns to proceed.
     */
    private void localNextTurn(int n) {
        for (int i = 0; i < n; i++) {
            players.addLast(players.removeFirst());
        }
    }

    /**
     * Finishes a player's participation in the game.
     *
     * @param p The player to finish.
     */
    private void finishPlayer(Player p) {
        ranking.addLast(players.removeLast());
    }

    /**
     * Returns the current player whose turn it is.
     *
     * @return The current player.
     */
    Player getCurrentPlayer() {
        return players.isEmpty() ? null : players.getFirst();
    }
}
